@@ -687,11 +687,12 @@ class KojiContentGenerator(object):
         mmd = self._sanitize_mmd(self.module.mmd())
         if self.devel:
             # Set the new name
-            mmd = mmd.copy(mmd.get_module_name() + "-devel")
+            orig_name = mmd.get_module_name()
+            mmd = mmd.copy(orig_name + "-devel")
 
             # Depend on the actual module
             for dep in mmd.get_dependencies():
-                dep.add_runtime_stream(mmd.get_module_name(), mmd.get_stream_name())
+                dep.add_runtime_stream(orig_name, mmd.get_stream_name())
 
             # Delete API and profiles
             for rpm in mmd.get_rpm_api():
