@@ -1,27 +1,41 @@
 # -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MIT
-from __future__ import print_function
-from flask_script import Manager, prompt_bool
+from __future__ import absolute_import, print_function
 from functools import wraps
-import flask_migrate
+import getpass
 import logging
 import os
-import getpass
 import textwrap
 
+import flask_migrate
+from flask_script import Manager, prompt_bool
 from werkzeug.datastructures import FileStorage
-from module_build_service import app, conf, db, create_app
-from module_build_service import models
-from module_build_service.utils import (
-    submit_module_build_from_yaml,
-    load_local_builds,
-    load_mmd_file,
-    import_mmd,
-    import_builds_from_local_dnf_repos,
+
+from module_build_service import app, db
+from module_build_service.builder.MockModuleBuilder import (
+    import_builds_from_local_dnf_repos, load_local_builds
 )
-from module_build_service.errors import StreamAmbigous
-import module_build_service.messaging
+from module_build_service.common import conf, models
+from module_build_service.common.errors import StreamAmbigous
+from module_build_service.common.logger import level_flags
+from module_build_service.common.utils import load_mmd_file, import_mmd
 import module_build_service.scheduler.consumer
+from module_build_service.scheduler.db_session import db_session
+import module_build_service.scheduler.local
+from module_build_service.web.submit import submit_module_build_from_yaml
+
+
+def create_app(debug=False, verbose=False, quiet=False):
+    # logging (intended for flask-script, see manage.py)
+    log = logging.getLogger(__name__)
+    if debug:
+        log.setLevel(level_flags["debug"])
+    elif verbose:
+        log.setLevel(level_flags["verbose"])
+    elif quiet:
+        log.setLevel(level_flags["quiet"])
+
+    return app
 
 
 manager = Manager(create_app)
@@ -124,7 +138,7 @@ def build_module_locally(
     if "SERVER_NAME" not in app.config or not app.config["SERVER_NAME"]:
         app.config["SERVER_NAME"] = "localhost"
 
-        if app.config["RESOLVER"] == "db":
+        if conf.resolver == "db":
             raise ValueError(
                 "Please set RESOLVER to 'mbs' in your configuration for local builds.")
 
@@ -143,6 +157,9 @@ def build_module_locally(
         os.remove(dbpath)
 
     db.create_all()
+    # Reconfigure the backend database session registry to use the new the database location
+    db_session.remove()
+    db_session.configure(bind=db.session.bind)
 
     params = {
         "local_build": True,
@@ -157,37 +174,32 @@ def build_module_locally(
 
     yaml_file_path = os.path.abspath(yaml_file)
 
-    with models.make_db_session(conf) as db_session:
-        if offline:
-            import_builds_from_local_dnf_repos(db_session, platform_id)
-        load_local_builds(db_session, local_build_nsvs)
+    if offline:
+        import_builds_from_local_dnf_repos(platform_id)
+    load_local_builds(local_build_nsvs)
 
-        with open(yaml_file_path) as fd:
-            filename = os.path.basename(yaml_file)
-            handle = FileStorage(fd)
-            handle.filename = filename
-            try:
-                module_builds = submit_module_build_from_yaml(
-                    db_session, username, handle, params,
-                    stream=str(stream), skiptests=skiptests
-                )
-            except StreamAmbigous as e:
-                logging.error(str(e))
-                logging.error("Use '-s module_name:module_stream' to choose the stream")
-                return
+    with open(yaml_file_path) as fd:
+        filename = os.path.basename(yaml_file)
+        handle = FileStorage(fd)
+        handle.filename = filename
+        try:
+            module_builds = submit_module_build_from_yaml(
+                db_session, username, handle, params,
+                stream=str(stream), skiptests=skiptests
+            )
+        except StreamAmbigous as e:
+            logging.error(str(e))
+            logging.error("Use '-s module_name:module_stream' to choose the stream")
+            return
 
-            module_build_ids = [build.id for build in module_builds]
+        module_build_ids = [build.id for build in module_builds]
 
-        stop = module_build_service.scheduler.make_simple_stop_condition(db_session)
+    module_build_service.scheduler.local.main(module_build_ids)
 
-    # Run the consumer until stop_condition returns True
-    module_build_service.scheduler.main([], stop)
-
-    with models.make_db_session(conf) as db_session:
-        has_failed_module = db_session.query(models.ModuleBuild).filter(
-            models.ModuleBuild.id.in_(module_build_ids),
-            models.ModuleBuild.state == models.BUILD_STATES["failed"],
-        ).count() > 0
+    has_failed_module = db_session.query(models.ModuleBuild).filter(
+        models.ModuleBuild.id.in_(module_build_ids),
+        models.ModuleBuild.state == models.BUILD_STATES["failed"],
+    ).count() > 0
 
     if has_failed_module:
         raise RuntimeError("Module build failed")
@@ -221,28 +233,29 @@ def retire(identifier, confirm=False):
     if len(parts) >= 4:
         filter_by_kwargs["context"] = parts[3]
 
-    with models.make_db_session(conf) as db_session:
-        # Find module builds to retire
-        module_builds = db_session.query(models.ModuleBuild).filter_by(**filter_by_kwargs).all()
+    # Find module builds to retire
+    module_builds = db_session.query(models.ModuleBuild).filter_by(**filter_by_kwargs).all()
 
-        if not module_builds:
-            logging.info("No module builds found.")
-            return
+    if not module_builds:
+        logging.info("No module builds found.")
+        return
 
-        logging.info("Found %d module builds:", len(module_builds))
-        for build in module_builds:
-            logging.info("\t%s", ":".join((build.name, build.stream, build.version, build.context)))
+    logging.info("Found %d module builds:", len(module_builds))
+    for build in module_builds:
+        logging.info("\t%s", ":".join((build.name, build.stream, build.version, build.context)))
 
-        # Prompt for confirmation
-        is_confirmed = confirm or prompt_bool("Retire {} module builds?".format(len(module_builds)))
-        if not is_confirmed:
-            logging.info("Module builds were NOT retired.")
-            return
+    # Prompt for confirmation
+    is_confirmed = confirm or prompt_bool("Retire {} module builds?".format(len(module_builds)))
+    if not is_confirmed:
+        logging.info("Module builds were NOT retired.")
+        return
 
-        # Retire module builds
-        for build in module_builds:
-            build.transition(
-                db_session, conf, models.BUILD_STATES["garbage"], "Module build retired")
+    # Retire module builds
+    for build in module_builds:
+        build.transition(
+            db_session, conf, models.BUILD_STATES["garbage"], "Module build retired")
+
+    db_session.commit()
 
     logging.info("Module builds retired.")
 
