@@ -6,7 +6,6 @@ from datetime import datetime
 import math
 import os
 import re
-import time
 
 from gi.repository import GLib
 import requests
@@ -16,7 +15,8 @@ from module_build_service.common.errors import Conflict, Forbidden, ValidationEr
 from module_build_service.common.messaging import notify_on_module_state_change
 from module_build_service.common.modulemd import Modulemd
 from module_build_service.common.submit import fetch_mmd
-from module_build_service.common.utils import load_mmd, mmd_to_str, to_text_type
+from module_build_service.common.utils import (load_mmd, mmd_to_str, to_text_type,
+                                               provide_module_stream_version_from_mmd)
 from module_build_service.web.mse import generate_expanded_mmds, generate_mmds_from_static_contexts
 from module_build_service.web.utils import deps_to_dict
 
@@ -47,7 +47,7 @@ def validate_mmd(mmd):
         if name not in conf.allowed_privileged_module_names:
             raise ValidationError('The "mbs" xmd field is reserved for MBS')
 
-        allowed_keys = ["disttag_marking", "koji_tag_arches"]
+        allowed_keys = ["disttag_marking", "koji_tag_arches", 'static_context']
         for key in xmd["mbs"].keys():
             if key not in allowed_keys:
                 raise ValidationError('The "mbs" xmd field is reserved for MBS')
@@ -112,7 +112,6 @@ def submit_module_build_from_yaml(
 ):
     yaml_file = to_text_type(handle.read())
     mmd = load_mmd(yaml_file)
-    dt = datetime.utcfromtimestamp(int(time.time()))
     if hasattr(handle, "filename"):
         def_name = str(os.path.splitext(os.path.basename(handle.filename))[0])
     elif not mmd.get_module_name():
@@ -120,19 +119,20 @@ def submit_module_build_from_yaml(
             "The module's name was not present in the modulemd file. Please use the "
             '"module_name" parameter'
         )
-    def_version = int(dt.strftime("%Y%m%d%H%M%S"))
     module_name = mmd.get_module_name() or def_name
     module_stream = stream or mmd.get_stream_name() or "master"
     if module_name != mmd.get_module_name() or module_stream != mmd.get_stream_name():
         # This is how you set the name and stream in the modulemd
         mmd = mmd.copy(module_name, module_stream)
-    mmd.set_version(mmd.get_version() or def_version)
     if skiptests:
         buildopts = mmd.get_buildopts() or Modulemd.Buildopts()
         macros = buildopts.get_rpm_macros() or ""
         buildopts.set_rpm_macros(macros + "\n\n%__spec_check_pre exit 0\n")
         mmd.set_buildopts(buildopts)
-    return submit_module_build(db_session, username, mmd, params)
+
+    module_stream_version = provide_module_stream_version_from_mmd(mmd)
+
+    return submit_module_build(db_session, username, mmd, params, module_stream_version)
 
 
 _url_check_re = re.compile(r"^[^:/]+:.*$")
@@ -148,7 +148,9 @@ def submit_module_build_from_scm(db_session, username, params, allow_local_url=F
         url = "file://" + url
     mmd, scm = fetch_mmd(url, branch, allow_local_url)
 
-    return submit_module_build(db_session, username, mmd, params)
+    module_stream_version = int(scm.version)
+
+    return submit_module_build(db_session, username, mmd, params, module_stream_version)
 
 
 def _apply_dep_overrides(mmd, params):
@@ -387,6 +389,8 @@ def _process_support_streams(db_session, mmd, params):
 
         try:
             pp_rv = requests.get(schedule_url, timeout=15)
+            # raise exception if we receive 404
+            pp_rv.raise_for_status()
             pp_json = pp_rv.json()
             # Catch requests failures and JSON parsing errors
         except (requests.exceptions.RequestException, ValueError):
@@ -519,7 +523,7 @@ def _process_support_streams(db_session, mmd, params):
     _modify_buildtime_streams(db_session, mmd, new_streams_func)
 
 
-def submit_module_build(db_session, username, mmd, params):
+def submit_module_build(db_session, username, mmd, params, module_stream_version):
     """
     Submits new module build.
 
@@ -530,13 +534,6 @@ def submit_module_build(db_session, username, mmd, params):
     :rtype: list with ModuleBuild
     :return: List with submitted module builds.
     """
-    log.debug(
-        "Submitted %s module build for %s:%s:%s",
-        ("scratch" if params.get("scratch", False) else "normal"),
-        mmd.get_module_name(),
-        mmd.get_stream_name(),
-        mmd.get_version(),
-    )
 
     raise_if_stream_ambigous = False
     default_streams = {}
@@ -548,11 +545,18 @@ def submit_module_build(db_session, username, mmd, params):
     if "default_streams" in params:
         default_streams = params["default_streams"]
 
-    xmd = mmd.get_xmd()
-    # we check if static contexts are enabled by the `contexts` property defined by the user i
-    # as an build option.
-    static_context = "mbs_options" in xmd and "contexts" in xmd["mbs_options"]
-    input_mmds = generate_mmds_from_static_contexts(mmd) if static_context else [mmd]
+    input_mmds, static_context = process_module_context_configuration(mmd)
+
+    for mmd in input_mmds:
+        mmd.set_version(module_stream_version)
+
+    log.debug(
+        "Submitted %s module build for %s:%s:%s",
+        ("scratch" if params.get("scratch", False) else "normal"),
+        input_mmds[0].get_module_name(),
+        input_mmds[0].get_stream_name(),
+        input_mmds[0].get_version(),
+    )
 
     mmds = []
     for mmd in input_mmds:
@@ -688,3 +692,62 @@ def submit_module_build(db_session, username, mmd, params):
         raise Conflict(err_msg)
 
     return modules
+
+
+def process_module_context_configuration(mmd):
+    """
+    Processes initial module metadata context configurations and creates individual module
+    metadata for each context, if static context configuration is present.
+
+    :param Modulemd.ModuleStream packager_mmd: Packager (initial) modulemd which kickstarts
+        the build.
+    :rtype: list with ModuleBuild
+    :return: list of generated module metadata from context configurations.
+    """
+    mdversion = mmd.get_mdversion()
+    static_context = False
+
+    # we check what version of the metadata format we are using.
+    if mdversion == 3:
+        # v3 we always talking about a new build and the static context
+        # will be always True
+        static_context = True
+        mdindex = mmd.convert_to_index()
+        streams = mdindex.search_streams()
+
+        for stream in streams:
+            if not stream.is_static_context():
+                stream.set_static_context()
+
+            # we get the dependenices of the stream
+            deps = stream.get_dependencies()
+            # with v3 packager format the output v2 stream will always have
+            # only one set of dependecies. We need to remove the platform
+            # virtual module from runtime dependencies as it is not desired.
+            modules = deps[0].get_runtime_modules()
+            module_streams = [(m, deps[0].get_runtime_streams(m)[0]) for m in modules
+                              if m not in conf.base_module_names]
+            deps[0].clear_runtime_dependencies()
+
+            for module_stream in module_streams:
+                module, stream = module_stream
+                deps[0].add_runtime_stream(module, stream)
+
+        return streams, static_context
+    else:
+        xmd = mmd.get_xmd()
+        # check if we are handling rebuild of a static context module
+        if "mbs" in xmd:
+            # check if it is a static context
+            if "static_context" in xmd["mbs"] or mmd.is_static_context():
+                static_context = True
+                return [mmd], static_context
+
+        # we check if static contexts are enabled by the `contexts` property defined by the user i
+        # as an build option.
+        static_context = "mbs_options" in xmd and "contexts" in xmd["mbs_options"]
+        # if the static context configuration exists we expand it. If not we just return
+        # the mmd unchanged, for futher processing.
+        streams = generate_mmds_from_static_contexts(mmd) if static_context else [mmd]
+
+        return streams, static_context
