@@ -1,67 +1,52 @@
 # -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MIT
+from __future__ import absolute_import
+import os
+
+import koji
 import mock
 from mock import patch
-import module_build_service.messaging
-import module_build_service.scheduler.handlers.modules
-import os
-import koji
 import pytest
-from tests import conf, scheduler_init_data, read_staged_data
+
+from module_build_service.common import build_logs, conf
+from module_build_service.common.models import ComponentBuild, ModuleBuild
+from module_build_service.common.modulemd import Modulemd
 import module_build_service.resolver
-from module_build_service import build_logs, Modulemd
-from module_build_service.utils.general import load_mmd
-from module_build_service.models import ComponentBuild, ModuleBuild
+from module_build_service.scheduler.db_session import db_session
+import module_build_service.scheduler.handlers.modules
+from tests import scheduler_init_data
 
 base_dir = os.path.dirname(os.path.dirname(__file__))
 
 
 class TestModuleWait:
     def setup_method(self, test_method):
+        scheduler_init_data()
+
         self.config = conf
         self.session = mock.Mock()
-        self.fn = module_build_service.scheduler.handlers.modules.wait
 
     def teardown_method(self, test_method):
         try:
-            with module_build_service.models.make_db_session(conf) as db_session:
-                path = build_logs.path(db_session, 1)
+            path = build_logs.path(db_session, 1)
             os.remove(path)
         except Exception:
             pass
 
     @patch("module_build_service.builder.GenericBuilder.create_from_module")
-    @patch("module_build_service.models.ModuleBuild.from_module_event")
-    def test_init_basic(self, from_module_event, create_builder):
+    def test_init_basic(self, create_builder):
         builder = mock.Mock()
         builder.get_disttag_srpm.return_value = "some srpm disttag"
         builder.build.return_value = 1234, 1, "", None
         builder.module_build_tag = {"name": "some-tag-build"}
         create_builder.return_value = builder
-        mocked_module_build = mock.Mock()
-        mocked_module_build.name = "foo"
-        mocked_module_build.stream = "stream"
-        mocked_module_build.version = "1"
-        mocked_module_build.context = "1234567"
-        mocked_module_build.state = 1
-        mocked_module_build.id = 1
-        mocked_module_build.json.return_value = {
-            "name": "foo",
-            "stream": "1",
-            "version": 1,
-            "state": "some state",
-            "id": 1,
-        }
-        mocked_module_build.id = 1
-        mocked_module_build.mmd.return_value = load_mmd(read_staged_data("formatted_testmodule"))
-        mocked_module_build.component_builds = []
 
-        from_module_event.return_value = mocked_module_build
-
-        msg = module_build_service.messaging.MBSModule(
-            msg_id=None, module_build_id=1, module_build_state="some state")
+        module_build_id = db_session.query(ModuleBuild).first().id
         with patch("module_build_service.resolver.GenericResolver.create"):
-            self.fn(config=self.config, db_session=self.session, msg=msg)
+            module_build_service.scheduler.handlers.modules.wait(
+                msg_id="msg-id-1",
+                module_build_id=module_build_id,
+                module_build_state="some state")
 
     @patch(
         "module_build_service.builder.GenericBuilder.default_buildroot_groups",
@@ -71,12 +56,11 @@ class TestModuleWait:
     @patch("module_build_service.resolver.DBResolver")
     @patch("module_build_service.resolver.GenericResolver")
     def test_new_repo_called_when_macros_reused(
-        self, generic_resolver, resolver, create_builder, dbg, db_session
+        self, generic_resolver, resolver, create_builder, dbg
     ):
         """
         Test that newRepo is called when module-build-macros build is reused.
         """
-        scheduler_init_data(db_session)
         koji_session = mock.MagicMock()
         koji_session.newRepo.return_value = 123456
 
@@ -97,11 +81,10 @@ class TestModuleWait:
         resolver.get_module_tag.return_value = "module-testmodule-master-20170109091357"
 
         generic_resolver.create.return_value = resolver
-        msg = module_build_service.messaging.MBSModule(
-            msg_id=None, module_build_id=2, module_build_state="some state")
 
         module_build_service.scheduler.handlers.modules.wait(
-            config=conf, db_session=db_session, msg=msg)
+            msg_id="msg-id-1",
+            module_build_id=2, module_build_state="some state")
 
         koji_session.newRepo.assert_called_once_with("module-123-build")
 
@@ -119,12 +102,11 @@ class TestModuleWait:
     @patch("module_build_service.resolver.DBResolver")
     @patch("module_build_service.resolver.GenericResolver")
     def test_new_repo_not_called_when_macros_not_reused(
-        self, generic_resolver, resolver, create_builder, dbg, db_session
+        self, generic_resolver, resolver, create_builder, dbg
     ):
         """
         Test that newRepo is called everytime for module-build-macros
         """
-        scheduler_init_data(db_session)
         koji_session = mock.MagicMock()
         koji_session.newRepo.return_value = 123456
 
@@ -145,11 +127,11 @@ class TestModuleWait:
         resolver.get_module_tag.return_value = "module-testmodule-master-20170109091357"
 
         generic_resolver.create.return_value = resolver
-        msg = module_build_service.messaging.MBSModule(
-            msg_id=None, module_build_id=2, module_build_state="some state")
 
         module_build_service.scheduler.handlers.modules.wait(
-            config=conf, db_session=db_session, msg=msg)
+            msg_id="msg-id-1",
+            module_build_id=2,
+            module_build_state="some state")
 
         assert koji_session.newRepo.called
 
@@ -161,14 +143,13 @@ class TestModuleWait:
     @patch("module_build_service.resolver.DBResolver")
     @patch("module_build_service.resolver.GenericResolver")
     def test_set_cg_build_koji_tag_fallback_to_default(
-        self, generic_resolver, resolver, create_builder, dbg, db_session
+        self, generic_resolver, resolver, create_builder, dbg
     ):
         """
         Test that build.cg_build_koji_tag fallbacks to default tag.
         """
         base_mmd = Modulemd.ModuleStreamV2.new("base-runtime", "f27")
 
-        scheduler_init_data(db_session)
         koji_session = mock.MagicMock()
         koji_session.newRepo.return_value = 123456
 
@@ -192,11 +173,11 @@ class TestModuleWait:
         }
 
         generic_resolver.create.return_value = resolver
-        msg = module_build_service.messaging.MBSModule(
-            msg_id=None, module_build_id=2, module_build_state="some state")
 
         module_build_service.scheduler.handlers.modules.wait(
-            config=conf, db_session=db_session, msg=msg)
+            msg_id="msg-id-1",
+            module_build_id=2,
+            module_build_state="some state")
 
         module_build = ModuleBuild.get_by_id(db_session, 2)
         assert module_build.cg_build_koji_tag == "modular-updates-candidate"
@@ -216,7 +197,7 @@ class TestModuleWait:
     @patch("module_build_service.resolver.DBResolver")
     @patch("module_build_service.resolver.GenericResolver")
     @patch(
-        "module_build_service.config.Config.base_module_names",
+        "module_build_service.common.config.Config.base_module_names",
         new_callable=mock.PropertyMock,
         return_value=["base-runtime", "platform"],
     )
@@ -229,14 +210,12 @@ class TestModuleWait:
         dbg,
         koji_cg_tag_build,
         expected_cg_koji_build_tag,
-        db_session,
     ):
         """
         Test that build.cg_build_koji_tag is set.
         """
         base_mmd = Modulemd.ModuleStreamV2.new("base-runtime", "f27")
 
-        scheduler_init_data(db_session)
         koji_session = mock.MagicMock()
         koji_session.newRepo.return_value = 123456
 
@@ -265,11 +244,9 @@ class TestModuleWait:
             new=koji_cg_tag_build,
         ):
             generic_resolver.create.return_value = resolver
-            msg = module_build_service.messaging.MBSModule(
-                msg_id=None, module_build_id=2, module_build_state="some state"
-            )
             module_build_service.scheduler.handlers.modules.wait(
-                config=conf, db_session=db_session, msg=msg
-            )
+                msg_id="msg-id-1",
+                module_build_id=2,
+                module_build_state="some state")
             module_build = ModuleBuild.get_by_id(db_session, 2)
             assert module_build.cg_build_koji_tag == expected_cg_koji_build_tag
