@@ -12,15 +12,19 @@ from werkzeug.datastructures import FileStorage
 
 from module_build_service.common import models
 from module_build_service.common.errors import ValidationError
-from module_build_service.common.utils import mmd_to_str, load_mmd
+from module_build_service.common.utils import (mmd_to_str, load_mmd,
+                                               provide_module_stream_version_from_timestamp)
 from module_build_service.scheduler.db_session import db_session
 from module_build_service.web.submit import (
-    get_prefixed_version, submit_module_build, submit_module_build_from_yaml
+    get_prefixed_version, submit_module_build, submit_module_build_from_yaml,
+    process_module_context_configuration
 )
 from tests import (
     scheduler_init_data,
     make_module_in_db,
     make_module,
+    read_staged_data,
+    init_data,
 )
 
 
@@ -50,43 +54,12 @@ class TestSubmit:
         is set to False.
         """
 
-        yaml_str = """
-document: modulemd
-version: 2
-data:
-    name: app
-    stream: test
-    summary: "A test module"
-    description: >
-        "A test module stream"
-    license:
-        module: [ MIT ]
-    dependencies:
-        - buildrequires:
-            platform: []
-            gtk: []
-          requires:
-            platform: []
-            gtk: []
-    xmd:
-        mbs_options:
-            contexts:
-                context1:
-                    buildrequires:
-                        platform: f28
-                    requires:
-                        platform: f28
-                        gtk: 1
-                context2:
-                    buildrequires:
-                        platform: f28
-                    requires:
-                        platform: f28
-                        gtk: 2
-        """
+        yaml_str = read_staged_data("static_context_v2")
         mmd = load_mmd(yaml_str)
 
-        builds = submit_module_build(db_session, "app", mmd, {})
+        ux_timestamp = "1613048427"
+        version = provide_module_stream_version_from_timestamp(ux_timestamp)
+        builds = submit_module_build(db_session, "app", mmd, {}, version)
 
         expected_context = ["context1", "context2"]
 
@@ -96,6 +69,7 @@ data:
             assert build.context in expected_context
             mmd = build.mmd()
             xmd = mmd.get_xmd()
+            assert mmd.get_version() == int("28" + str(version))
             assert "mbs_options" not in xmd
             assert xmd["mbs"]["static_context"]
 
@@ -105,38 +79,19 @@ data:
         are more options configured then `contexts` option..
         """
 
-        yaml_str = """
-document: modulemd
-version: 2
-data:
-    name: app
-    stream: test1
-    summary: "A test module"
-    description: >
-        "A test module stream"
-    license:
-        module: [ MIT ]
-    dependencies:
-        - buildrequires:
-            platform: []
-            gtk: []
-          requires:
-            platform: []
-            gtk: []
-    xmd:
-        mbs_options:
-            contexts:
-                context1:
-                    buildrequires:
-                        platform: f28
-                    requires:
-                        platform: f28
-                        gtk: 1
-            another_option: "test"
-        """
+        yaml_str = read_staged_data("static_context_v2")
         mmd = load_mmd(yaml_str)
+        xmd = mmd.get_xmd()
+        xmd["mbs_options"]["another_option"] = "test"
+        xmd["mbs_options"]["contexts"] = {
+            "context3": xmd["mbs_options"]["contexts"]["context1"]
+        }
+        mmd.set_xmd(xmd)
 
-        builds = submit_module_build(db_session, "app", mmd, {})
+        ux_timestamp = "1613048427"
+        version = provide_module_stream_version_from_timestamp(ux_timestamp)
+
+        builds = submit_module_build(db_session, "app", mmd, {}, version)
 
         assert len(builds) == 1
         mmd = builds[0].mmd()
@@ -145,6 +100,123 @@ data:
         assert "another_option" in xmd["mbs_options"]
         assert "test" == xmd["mbs_options"]["another_option"]
 
+    def test_submit_build_module_context_configurations(self):
+        """
+        With the introduction of the v3 packager yaml format we replace MSE with static contexts.
+        This test tests the submission of such a packager file.
+        """
+        init_data(multiple_stream_versions=True)
+        yaml_str = read_staged_data("v3/mmd_packager")
+        mmd = load_mmd(yaml_str)
+        ux_timestamp = "1613048427"
+        version = provide_module_stream_version_from_timestamp(ux_timestamp)
+        builds = submit_module_build(db_session, "foo", mmd, {}, version)
+
+        assert len(builds) == 2
+
+        expected_deps = {"CTX1": {"buildrequires": {"platform": ["f28"]},
+                                  "requires": {"nginx": ["1"]}},
+                         "CTX2": {"buildrequires": {"platform": ["f29.2.0"]},
+                                  "requires": {}}}
+
+        for build in builds:
+            mmd = build.mmd()
+            context = mmd.get_context()
+            assert context in expected_deps
+            assert mmd.get_mdversion() == 2
+
+            deps = mmd.get_dependencies()[0]
+
+            btms = deps.get_buildtime_modules()
+            rtms = deps.get_runtime_modules()
+
+            for btm in btms:
+                expected_stream = expected_deps[context]["buildrequires"][btm][0]
+                actual_stream = deps.get_buildtime_streams(btm)[0]
+                assert expected_stream == actual_stream
+
+            for rtm in rtms:
+                expected_stream = expected_deps[context]["requires"][rtm][0]
+                actual_stream = deps.get_runtime_streams(rtm)[0]
+                assert expected_stream == actual_stream
+
+            xmd = mmd.get_xmd()
+            assert "mbs_options" not in xmd
+            assert xmd["mbs"]["static_context"]
+
+
+class TestProcessModuleContextConfiguration:
+    """
+    With the introduction of static contexts, we have now several different configuration of
+    static contexts for different major/minor RHEL releases.
+    """
+
+    def test_process_mse_configuration(self):
+        """
+        Testing the processing of MSE type packager (v2) file i. e. no static context.
+        """
+
+        yaml_str = read_staged_data("static_context_v2")
+        mmd = load_mmd(yaml_str)
+        mmd.set_xmd({})
+
+        streams, static_context = process_module_context_configuration(mmd)
+
+        assert not static_context
+        assert len(streams) == 1
+        assert not mmd.get_context()
+
+    def test_process_initial_xmd_static_configuration(self):
+        """
+        Testing the processing of MSE type packager (v2) file with static context
+        configuration in XMD
+        """
+        yaml_str = read_staged_data("static_context_v2")
+        mmd = load_mmd(yaml_str)
+
+        streams, static_context = process_module_context_configuration(mmd)
+
+        assert static_context
+        assert len(streams) == 2
+        expected_contexts = ["context1", "context2"]
+        for stream in streams:
+            assert stream.get_context() in expected_contexts
+
+    def test_process_xmd_static_context_rebuild(self):
+        """
+        Testing the processing of MSE type file (v2) used for rebuild with set static_context in
+        mbs metadata in xmd.
+        """
+
+        yaml_str = read_staged_data("testmodule_v2")
+        mmd = load_mmd(yaml_str)
+        mmd.set_context("context1")
+        mmd.set_xmd({"mbs": {"static_context": True}})
+
+        streams, static_context = process_module_context_configuration(mmd)
+
+        assert static_context
+        assert len(streams) == 1
+        assert streams[0].get_context() == "context1"
+
+    def test_process_v3_packager_file(self):
+        """
+        Testing the processing of v3 packager file with static context configurations.
+        """
+
+        yaml_str = read_staged_data("v3/mmd_packager")
+        mmd = load_mmd(yaml_str)
+
+        streams, static_context = process_module_context_configuration(mmd)
+
+        assert static_context
+        assert len(streams) == 2
+        expected_contexts = ["CTX1", "CTX2"]
+        for stream in streams:
+            assert stream.get_mdversion() == 2
+            assert stream.get_context() in expected_contexts
+            assert stream.is_static_context()
+
 
 @pytest.mark.usefixtures("reuse_component_init_data")
 class TestUtilsComponentReuse:
@@ -200,10 +272,13 @@ class TestUtilsComponentReuse:
         mmd1_copy = mmd1.copy()
         mmd1_copy.set_xmd({})
 
-        builds = submit_module_build(db_session, "foo", mmd1_copy, {})
-        ret = {b.mmd().get_context(): b.state for b in builds}
-        assert ret == {"c1": models.BUILD_STATES["ready"], "c2": models.BUILD_STATES["init"]}
+        ux_timestamp = "1613048427"
+        version = provide_module_stream_version_from_timestamp(ux_timestamp)
 
+        builds = submit_module_build(db_session, "foo", mmd1_copy, {}, version)
+        ret = {b.mmd().get_context(): b.state for b in builds}
+
+        assert ret == {"c1": models.BUILD_STATES["ready"], "c2": models.BUILD_STATES["init"]}
         assert builds[0].siblings(db_session) == [builds[1].id]
         assert builds[1].siblings(db_session) == [builds[0].id]
 
@@ -223,10 +298,13 @@ class TestUtilsComponentReuse:
         mmd_copy = mmd.copy()
         mmd_copy.set_xmd({})
 
+        ux_timestamp = "1613048427"
+        version = provide_module_stream_version_from_timestamp(ux_timestamp)
+
         with pytest.raises(
             ValidationError,
             match="Only scratch module builds can be built from this branch.",
         ):
-            submit_module_build(db_session, "foo", mmd_copy, {"branch": "private-foo"})
+            submit_module_build(db_session, "foo", mmd_copy, {"branch": "private-foo"}, version)
 
-        submit_module_build(db_session, "foo", mmd_copy, {"branch": "otherbranch"})
+        submit_module_build(db_session, "foo", mmd_copy, {"branch": "otherbranch"}, version)
